//========================================================================//
//
// EXERCISE 132
// REVISED JAN 2025 (stpm3)
//
//========================================================================//


// Time-dependent effects in flexible parametric models
//Load and stset the data
frames reset
use melanoma if stage == 1, clear
gen female = sex == 2
stset surv_mm, failure(status==1) exit(time 60.5) scale(12)

// (a) fit a Cox model 
// Assess the PH assumption for age group
stcox i.female i.year8594 i.agegrp,
estat phtest, detail
forvalue i = 1/3 {
	local beta = _b[`i'.agegrp]
	estat phtest, plot(`i'.agegrp) name(sch_age`i', replace) ///
		yline(0 `beta') msize(small) mcolor(blue%30) msymbol(Oh) bw(0.4)	
}	

// (b) fit flexible parametric model
stpm3 i.female i.year8594 i.agegrp, df(4) scale(lncumhazard) eform
estimates store ph

predict h_age0 h_age1 h_age2 h_age3, ///
        hazard ci frame(f1, replace) ///
        timevar(0 10, step(0.1))     ///
        at1(agegrp 0) at2(agegrp 1)  ///
        at3(agegrp 2) at4(agegrp 3)  ///
        zeros per(1000)

frame f1 {
twoway (line h_age? tt) ///
       ,xtitle("Time since diagnosis (years)") ///
       ytitle("Cause specific mortality rate (per 1000 py's)") ///
       legend(order(1 "<45" 2 "45-59" 3 "60-74" 4 "75+") ring(0) pos(1) cols(1)) ///
       name(hazard_ph, replace)
}		

// compare fitted values to KM
// some evidence of a poor fit
stpm3km agegrp, factor name(stpmkm_ph, replace)

/* (c) Time-dependent effects for age group */
stpm3 i.female i.year8594 i.agegrp, df(4) scale(lncumhazard) ///
      tvc(i.agegrp) dftvc(2)
estimates store nonph

lrtest ph nonph

/* (d) predict the hazard for each age group */
predict h_age0_tvc h_age1_tvc h_age2_tvc h_age3_tvc, ///
        hazard ci frame(f1, merge)                   ///
        at1(agegrp 0) at2(agegrp 1)                  ///
        at3(agegrp 2) at4(agegrp 3)                  ///
        zeros per(1000)
        
frame f1 {
twoway (line h_age? tt) ///
       (line h_age?_tvc tt, pstyle(p1 p2 p3 p4) lpattern(dash..)) ///
       ,xtitle("Time since diagnosis (years)") ///
       ytitle("Cause specific mortality rate (per 1000 py's)") ///
       legend(order(1 "<45" 2 "45-59" 3 "60-74" 4 "75+") ring(0) pos(1) cols(1)) ///
       name(hazard_ph, replace)
}	        
        
        
/* (e) time-dependent log-hazard ratios */
predict , nogen                      ///
        hazard ci frame(f1, merge)   ///
        at1(agegrp 0) at2(agegrp 1)  ///
        at3(agegrp 2) at4(agegrp 3)  ///
        zeros                        ///
        contrast(ratio)              ///
        contrastvar(hr2 hr3 hr4)

frame f1 {
  twoway (line hr2 hr3 hr4 tt, sort), ///
          yscale(log)  ylabel(1 2 10 20 50) ///
          legend(order(1 "Age 45-59" 2 "Age 60-74" 3 "Age 75+") ring(0) pos(1) cols(1)) ///
          xtitle("Time since diagnosis (years)") ///
          ytitle("Hazard ratio") ///
          name(hr, replace)
    
   twoway  (line hr4 tt, sort) ///
           (rarea hr4_lci hr4_uci tt, sort color(%30) pstyle(p1line)) ///
           ,legend(off) yscale(log) ylabel(1 2 10 20 50) ///
           xtitle("Time since diagnosis (years)") ///
           ytitle("Hazard ratio") ///
           name(hr_age4, replace)    
}		

// (f) compare fitted values to KM
stpm3km agegrp, factor name(stpmkm_tvc, replace)


// (g) Difference in hazard rates 
predict , nogen                      ///
        hazard ci frame(f1, merge)   ///
        at1(agegrp 0) at2(agegrp 1)  ///
        at3(agegrp 2) at4(agegrp 3)  ///
        zeros per(1000)              ///
        contrast(difference)         ///
        contrastvar(hd2 hd3 hd4)

frame f1 {        
  twoway (line hd3 tt, sort) ///
         (rarea hd3_lci hd3_uci tt, sort pstyle(p1line) color(%30)) ///
         , legend(off)  ///
         title("Time since diagnosis (years)") ///
         ytitle("Difference in mortality rate (per 1000py)") ///
         name(hdiff, replace)
}
  
// (h) Difference in survival functions */
predict sage0 sage3, survival ci frame(f1, merge)  ///
        at1(agegrp 0 female 1 year8594 1)          ///
        at2(agegrp 3 female 1 year8594 1)          ///
        zeros                                      ///
        contrast(difference)                       ///
        contrastvar(sdiff)

frame f1 {
  twoway	line sage0 sage3 tt, sort                             ///
          xtitle("Time since diagnosis (years)")                ///
          ytitle("S(t)")                                        ///
          legend(order(1 "<45" 2 "75+") ring(0) pos(1) cols(1)) ///
          name(surv_old_young, replace)

  twoway (line sdiff tt, sort)                                     ///
         (rarea sdiff_lci sdiff_uci tt, pstyle(p1line) color(%30)) ///
  		   ,legend(off)                                              ///
  		   xtitle("Time since diagnosis (years)")                    ///
  		   ytitle("Difference in survival functions")                ///
  		   name(sdiff, replace)
}


// (i) varying df for time-dependent effects 
forvalues i = 1/3 {
  stpm3 i.female year8594 i.agegrp, df(4) scale(lncumhazard) ///
        tvc(i.agegrp) dftvc(`i')
  estimates store dftvc`i'
  predict  , nogen hazard ci frame(hr, mergecreate) ///
             timevar(0 10, step(0.1))               ///
             at1(agegrp 0, zeros)                   ///
             at2(agegrp 3, zeros)                   ///
             contrast(ratio)                        ///
             contrastvar(hr`i')
}

count if _d==1
estimates stats dftvc*, n(`r(N)')

frame hr {
  local linestuff lpattern(solid dash dash) lwidth(medthick thin thin)
  twoway	(line hr1 hr1_lci hr1_uci tt, sort lcolor(red..)  `linestuff') ///
  		(line hr2 hr2_lci hr2_uci tt, sort lcolor(midblue..)  `linestuff') ///
  		(line hr3 hr3_lci hr3_uci tt, sort lcolor(midgreen..) `linestuff') ///
  		if tt>0.1, ///
  		yscale(log) ///
  		ylabel(1 2 4 8 20 50, angle(h)) ///
  		legend(order(1 "1 df" 4 "2 df" 7 "3 df") ring(0) pos(1) cols(1)) ///
  		xtitle("Time since diagnosis (years)") ///
  		ytitle("Hazard Ratio") ///
  		yscale(log) ///
  		name(tvc_df_comp, replace)
}		
// (j) two time-dependent effects */
stpm3 i.female  i.agegrp, df(4) scale(lncumhazard) ///
	tvc(i.agegrp i.female) dftvc(3)
  
predict , nogen hazard ci frame(hrcomp, replace) ///
          timevar(0.1 10, step(0.1)) ///
          at1(female 0 agegrp 0) ///
          at2(female 1 agegrp 0) ///
          contrast(ratio)        ///
          contrastvar(hr_f_age0)
          
predict , nogen hazard ci frame(hrcomp, merge) ///
          at1(female 0 agegrp 3) ///
          at2(female 1 agegrp 3) ///
          contrast(ratio)        ///
          contrastvar(hr_f_age3)          

frame hrcomp {
  twoway (line hr_f_age0* hr_f_age3* tt if tt>0.1, sort yscale(log))	
}	

// (k) note if we fit a model on the log hazard scale - we do not get this problem
// but the models take longer to fit as we need to use
// numerical integration when fitting the model.		

stpm3 i.female  i.agegrp, df(4) scale(lnhazard) ///
	tvc(i.agegrp i.female) dftvc(2) splinetype(ns)    

  
predict , nogen hazard ci frame(hrcomp2, replace) ///
          timevar(0.1 10, step(0.1)) ///
          at1(female 0 agegrp 0) ///
          at2(female 1 agegrp 0) ///
          contrast(ratio)        ///
          contrastvar(hr_f_age0)
          
predict , nogen hazard ci frame(hrcomp2, merge) ///
          at1(female 0 agegrp 3) ///
          at2(female 1 agegrp 3) ///
          contrast(ratio)        ///
          contrastvar(hr_f_age3)          

frame hrcomp2 {
  twoway (line hr_f_age0* hr_f_age3* tt if tt>0.1, sort yscale(log))	
}	

// Solutions

// (a) Some evdience of non-proportional effects - notably for oldest age group.	

// (b) some disagreement in survival for teh oldest age group.

// (c) Fairly strong evidence of non-proportional effect from LR test.

// (e) The hazard ratio is high for the oldest age group. If you look
//     at the hazard rate in the youngest age group, this is very low just
//     after follow-up. Thus the denominator is small leading to a large ratio.
		
// (f) Evidence of a better fit, particularly for the oldest age group.


// (g) The hazard difference is small as the hazard rates are low just after 
//     the start of follow-up. So even though the ratio is large it is
//     applied to rates that are very low.

// (i) Fairly good agreement. AIC/BIC selects 1df.