// Code for lecture on Modelling (Biostat III)
// 2025-01-05 Anna Johansson

use melanoma, clear

describe

* generate _d (used in .poisson below)
stset surv_mm, fail(status==1,2) scale(12)


// Categorise age

tab age
tab agegrp
codebook agegrp

gen old=1 if age>=60
replace old=0 if age<60
tab old

egen age3=cut(age), at(0,60,75,100)
tab age3

/*
. tab old

        old |      Freq.     Percent        Cum.
------------+-----------------------------------
          0 |      4,284       55.10       55.10
          1 |      3,491       44.90      100.00
------------+-----------------------------------
      Total |      7,775      100.00


. tab age3

       age3 |      Freq.     Percent        Cum.
------------+-----------------------------------
          0 |      4,284       55.10       55.10
         60 |      2,280       29.32       84.42
         75 |      1,211       15.58      100.00
------------+-----------------------------------
      Total |      7,775      100.00
*/

* rate
strate age3, per(1000)

di 81.8/32.8 // 2.49
di 191.4/32.8 // 5.84
/*
  +-----------------------------------------------------+
  | age3      D         Y      Rate     Lower     Upper |
  |-----------------------------------------------------|
  |    0   1094   33.3592    32.795    30.908    34.797 |
  |   60   1101   13.4584    81.808    77.115    86.785 |
  |   75    852    4.4521   191.369   178.941   204.660 |
  +-----------------------------------------------------+
*/


// Generate dummy variables X1, X2, X3

tabulate age3, generate(X)


// Poisson model

poisson _d X2 X3, exp(surv_mm) irr

/*
. poisson _d X2 X3, exp(surv_mm) irr

Iteration 0:  Log likelihood =  -8292.553  
Iteration 1:  Log likelihood = -8292.4919  
Iteration 2:  Log likelihood = -8292.4919  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(2)    = 1395.43
                                                       Prob > chi2   =  0.0000
Log likelihood = -8292.4919                            Pseudo R2     =  0.0776

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
          X2 |   2.494544   .1064893    21.41   0.000     2.294322     2.71224
          X3 |   5.835393   .2666325    38.60   0.000      5.33552    6.382098
       _cons |   .0027329   .0000826  -195.23   0.000     .0025756    .0028997
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.
*/


// Automatic generation of indicators using factor variables

poisson _d i.age3, exposure(surv_mm) irr

/*
. poisson _d i.age3, exposure(surv_mm) irr

Iteration 0:  Log likelihood =  -8292.553  
Iteration 1:  Log likelihood = -8292.4919  
Iteration 2:  Log likelihood = -8292.4919  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(2)    = 1395.43
                                                       Prob > chi2   =  0.0000
Log likelihood = -8292.4919                            Pseudo R2     =  0.0776

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
        age3 |
         60  |   2.494544   .1064893    21.41   0.000     2.294322     2.71224
         75  |   5.835393   .2666325    38.60   0.000      5.33552    6.382098
             |
       _cons |   .0027329   .0000826  -195.23   0.000     .0025756    .0028997
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.

*/


// Metric (continous) exposure variables

poisson _d age , exposure(surv_mm) irr

/*
. poisson _d age , exposure(surv_mm) irr

Iteration 0:  Log likelihood = -8204.4666  
Iteration 1:  Log likelihood = -8204.4645  
Iteration 2:  Log likelihood = -8204.4645  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(1)    = 1571.49
                                                       Prob > chi2   =  0.0000
Log likelihood = -8204.4645                            Pseudo R2     =  0.0874

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
         age |   1.048671   .0013184    37.80   0.000      1.04609    1.051258
       _cons |   .0003121   .0000256   -98.47   0.000     .0002658    .0003665
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.

. 
*/


gen age10=age/10
poisson _d age10, exposure(surv_mm) irr

/*
. poisson _d age10, exposure(surv_mm) irr

Iteration 0:  Log likelihood = -8204.4666  
Iteration 1:  Log likelihood = -8204.4645  
Iteration 2:  Log likelihood = -8204.4645  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(1)    = 1571.49
                                                       Prob > chi2   =  0.0000
Log likelihood = -8204.4645                            Pseudo R2     =  0.0874

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
       age10 |   1.608398   .0202208    37.80   0.000      1.56925    1.648522
       _cons |   .0003121   .0000256   -98.47   0.000     .0002658    .0003665
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.

*/

di 1.04867^10 // 1.6083792




// Main effects model using Poisson regression

poisson _d ib1.sex ib0.age3, exposure(surv_mm) irr baselevels

/*
. poisson _d ib1.sex ib0.age3, exposure(surv_mm) irr baselevels

Iteration 0:  Log likelihood = -8203.3551  
Iteration 1:  Log likelihood = -8203.2933  
Iteration 2:  Log likelihood = -8203.2933  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(3)    = 1573.83
                                                       Prob > chi2   =  0.0000
Log likelihood = -8203.2933                            Pseudo R2     =  0.0875

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
         sex |
       Male  |          1  (base)
     Female  |   .6140348   .0224068   -13.36   0.000     .5716519    .6595599
             |
        age3 |
          0  |          1  (base)
         60  |   2.499009   .1066805    21.45   0.000     2.298428    2.717096
         75  |   6.184747   .2839234    39.69   0.000     5.652568    6.767029
             |
       _cons |   .0034875   .0001193  -165.36   0.000     .0032612    .0037294
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.


*/

testparm 60.age3 75.age3

/*
. testparm 60.age3 75.age3

 ( 1)  [_d]60.age3 = 0
 ( 2)  [_d]75.age3 = 0

           chi2(  2) = 1586.33
         Prob > chi2 =    0.0000


*/

// Generate dummy 0/1 for sex
gen female=1 if sex==2
replace female=0 if sex==1
label define femlab 1 "Female" 0 "Male"
label values female femlab


poisson _d ib0.female ib0.age3, exposure(surv_mm)
poisson _d ib0.female ib0.age3, exposure(surv_mm) irr

/*
. poisson _d ib0.female ib0.age3, exposure(surv_mm)

Iteration 0:  Log likelihood = -8203.3551  
Iteration 1:  Log likelihood = -8203.2933  
Iteration 2:  Log likelihood = -8203.2933  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(3)    = 1573.83
                                                       Prob > chi2   =  0.0000
Log likelihood = -8203.2933                            Pseudo R2     =  0.0875

------------------------------------------------------------------------------
          _d | Coefficient  Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
      female |
     Female  |  -.4877037   .0364911   -13.36   0.000    -.5592251   -.4161824
             |
        age3 |
         60  |   .9158944   .0426891    21.45   0.000     .8322253    .9995635
         75  |   1.822086    .045907    39.69   0.000      1.73211    1.912062
             |
       _cons |  -5.658582   .0342201  -165.36   0.000    -5.725652   -5.591511
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------

. poisson _d ib0.female ib0.age3, exposure(surv_mm) irr

Iteration 0:  Log likelihood = -8203.3551  
Iteration 1:  Log likelihood = -8203.2933  
Iteration 2:  Log likelihood = -8203.2933  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(3)    = 1573.83
                                                       Prob > chi2   =  0.0000
Log likelihood = -8203.2933                            Pseudo R2     =  0.0875

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
      female |
     Female  |   .6140348   .0224068   -13.36   0.000     .5716519    .6595599
             |
        age3 |
         60  |   2.499009   .1066805    21.45   0.000     2.298428    2.717096
         75  |   6.184747   .2839234    39.69   0.000     5.652568    6.767029
             |
       _cons |   .0034875   .0001193  -165.36   0.000     .0032612    .0037294
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.

*/




di 0.0034875 * 0.6140348 * 2.499009  // .00535149

di 0.6140348 * 2.499009 // 1.5344785
di 0.6140348 * 6.184747 // 3.7976499




// Rates by sex and age

strate sex age3, per(1000)


/*
. strate sex age3, per(1000)

        Failure _d: status==1 2
  Analysis time _t: surv_mm/12

Estimated failure rates
Number of records = 7775

  +-------------------------------------------------------------+
  |    sex   age3     D         Y      Rate     Lower     Upper |
  |-------------------------------------------------------------|
  |   Male      0   685   14.6583    46.731    43.360    50.365 |
  |   Male     60   575    5.8649    98.041    90.346   106.391 |
  |   Male     75   341    1.4457   235.871   212.118   262.282 |
  | Female      0   409   18.7009    21.871    19.851    24.096 |
  | Female     60   526    7.5935    69.270    63.596    75.450 |
  |-------------------------------------------------------------|
  | Female     75   511    3.0064   169.970   155.854   185.365 |
  +-------------------------------------------------------------+
   Notes: Rate = D/Y = failures/person-time (per 1000).
          Lower and Upper are bounds of 95% confidence intervals.


*/

di 21.9/46.7 // 0.469
di 69.3/98.0 // 0.707
di 170.0/235.9 // 0.721


// Interaction model

poisson _d ib0.female##ib0.age3, exposure(surv_mm) irr

/*
. poisson _d ib0.female##ib0.age3, exposure(surv_mm) irr

Iteration 0:  Log likelihood = -8188.3986  
Iteration 1:  Log likelihood = -8188.3179  
Iteration 2:  Log likelihood = -8188.3179  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(5)    = 1603.78
                                                       Prob > chi2   =  0.0000
Log likelihood = -8188.3179                            Pseudo R2     =  0.0892

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
      female |
     Female  |    .468009   .0292453   -12.15   0.000     .4140604    .5289866
             |
        age3 |
         60  |   2.097968   .1186601    13.10   0.000     1.877825    2.343918
         75  |   5.047386   .3345166    24.43   0.000     4.432543    5.747514
             |
 female#age3 |
  Female#60  |   1.509675   .1311343     4.74   0.000     1.273345    1.789869
  Female#75  |   1.539727   .1443928     4.60   0.000     1.281208     1.85041
             |
       _cons |   .0038943   .0001488  -145.21   0.000     .0036133    .0041971
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.

*/

di 0.468 * 2.098 * 1.510  // 1.482 
di 0.468 * 5.047 * 1.540 // 3.637




// Testing for interactions

testparm 1.female#60.age3 1.female#75.age3


// Stratified effects

poisson _d ibn.age3 ib0.female#i.age3, exposure(surv_mm) irr nocons

/*
. poisson _d ibn.age3 ib0.female#i.age3, exposure(surv_mm) irr nocons

Iteration 0:  Log likelihood = -14447.429  
Iteration 1:  Log likelihood = -8198.1013  
Iteration 2:  Log likelihood = -8188.3477  
Iteration 3:  Log likelihood = -8188.3179  
Iteration 4:  Log likelihood = -8188.3179  

Poisson regression                                    Number of obs =    7,775
                                                      Wald chi2(6)  = 79148.24
Log likelihood = -8188.3179                           Prob > chi2   =   0.0000

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
        age3 |
          0  |   .0038943   .0001488  -145.21   0.000     .0036133    .0041971
         60  |   .0081701   .0003407  -115.27   0.000     .0075288    .0088659
         75  |   .0196559   .0010644   -72.56   0.000     .0176765    .0218569
             |
 female#age3 |
  Female# 0  |    .468009   .0292453   -12.15   0.000     .4140604    .5289866
  Female#60  |   .7065416   .0426289    -5.76   0.000     .6277414    .7952336
  Female#75  |   .7206063   .0503884    -4.69   0.000     .6283156    .8264532
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
*/

poisson _d ibn.female ib0.age3#i.female, exposure(surv_mm) irr nocons

/*
. poisson _d ibn.female ib0.age3#i.female, exposure(surv_mm) irr nocons

Iteration 0:  Log likelihood = -14447.429  
Iteration 1:  Log likelihood = -8198.1013  
Iteration 2:  Log likelihood = -8188.3477  
Iteration 3:  Log likelihood = -8188.3179  
Iteration 4:  Log likelihood = -8188.3179  

Poisson regression                                    Number of obs =    7,775
                                                      Wald chi2(6)  = 79148.24
Log likelihood = -8188.3179                           Prob > chi2   =   0.0000

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
      female |
       Male  |   .0038943   .0001488  -145.21   0.000     .0036133    .0041971
     Female  |   .0018226   .0000901  -127.56   0.000     .0016542     .002008
             |
 age3#female |
    60#Male  |   2.097968   .1186601    13.10   0.000     1.877825    2.343918
  60#Female  |    3.16725   .2088015    17.49   0.000     2.783343    3.604109
    75#Male  |   5.047386   .3345166    24.43   0.000     4.432543    5.747514
  75#Female  |   7.771598   .5156229    30.91   0.000     6.823946    8.850852
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------

*/


// Using lincom to obtain stratified effects

poisson _d ib0.female##ib0.age3, exposure(surv_mm) irr

/*
. poisson _d ib0.female##ib0.age3, exposure(surv_mm) irr

Iteration 0:  Log likelihood = -8188.3986  
Iteration 1:  Log likelihood = -8188.3179  
Iteration 2:  Log likelihood = -8188.3179  

Poisson regression                                     Number of obs =   7,775
                                                       LR chi2(5)    = 1603.78
                                                       Prob > chi2   =  0.0000
Log likelihood = -8188.3179                            Pseudo R2     =  0.0892

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
      female |
     Female  |    .468009   .0292453   -12.15   0.000     .4140604    .5289866
             |
        age3 |
         60  |   2.097968   .1186601    13.10   0.000     1.877825    2.343918
         75  |   5.047386   .3345166    24.43   0.000     4.432543    5.747514
             |
 female#age3 |
  Female#60  |   1.509675   .1311343     4.74   0.000     1.273345    1.789869
  Female#75  |   1.539727   .1443928     4.60   0.000     1.281208     1.85041
             |
       _cons |   .0038943   .0001488  -145.21   0.000     .0036133    .0041971
 ln(surv_mm) |          1  (exposure)
------------------------------------------------------------------------------
Note: _cons estimates baseline incidence rate.

*/


lincom 1.female + 1.female#60.age3, irr
lincom 1.female + 1.female#75.age3, irr

/*
. lincom 1.female + 1.female#60.age3, irr

 ( 1)  [_d]1.female + [_d]1.female#60.age3 = 0

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
         (1) |   .7065416   .0426289    -5.76   0.000     .6277414    .7952336
------------------------------------------------------------------------------

. lincom 1.female + 1.female#75.age3, irr

 ( 1)  [_d]1.female + [_d]1.female#75.age3 = 0

------------------------------------------------------------------------------
          _d |        IRR   Std. err.      z    P>|z|     [95% conf. interval]
-------------+----------------------------------------------------------------
         (1) |   .7206063   .0503884    -4.69   0.000     .6283155    .8264532
------------------------------------------------------------------------------

*/


// END OF FILE /////////////////////////////////////////////////////////////